<?php
/**
 * EPL Functions: Filemanager additions
 *
 * @link http://www.egroupware.org
 * @package epl
 * @subpackage filemanager
 * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
 * @copyright (c) 2010-16 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
 * @version $Id$
 */

use EGroupware\Api;
use EGroupware\Api\Link;
use EGroupware\Api\Framework;
use EGroupware\Api\Vfs;
use EGroupware\Api\Etemplate;

/**
 * Stylite filemanager additions:
 * - file a file dialog
 * - versioning admin (GUI mount tool) is in stylite_versioning_admin class
 */
class stylite_filemanager
{
	/**
	 * Methods callable via menuaction
	 *
	 * @var array
	 */
	var $public_functions = array(
		'upload' => true,
	);

	/**
	 * File a file dialog: Upload ability to set comment, custom fields and choose a path or entry to upload too
	 *
	 * @param array $content =null
	 * @param string $msg
	 */
	public static function upload(array $content=null, $msg = '')
	{
		$GLOBALS['egw_info']['flags']['currentapp'] = 'filemanager';	// to get the currect custom fields
		Api\Translation::add_app('filemanager');
		Framework::includeJS('','app','filemanager');

		if (!is_array($content))
		{
			$content = array();
			if (isset($_GET['path']))
			{
				$path = $_GET['path'];
			}
			elseif(isset($_GET['app']) && isset($GLOBALS['egw_info']['user']['apps'][$_GET['path']]))
			{
				$content['entry']['app'] = $_GET['app'];
				if (isset($_GET['id'])) $content['entry']['id'] = $_GET['id'];
				$path = Link::vfs_path($_GET['app'],$_GET['id'],'',true);
			}
			if (!isset($path) || !self::check_dir_writable($path, $msg))
			{
				if (!($nm = Api\Cache::getSession('filemanager', 'index')) || !($path = $nm['path']))
				{
					$path = filemanager_ui::get_home_dir();
				}
				self::check_dir_writable($path, $msg);
			}
			$content['path'] = Vfs::decodePath($path);
		}
		else
		{
			//_debug_array($content);
			$path = Vfs::encodePath($content['path']);
			if ($content['entry'] && $content['entry']['app'] && $content['entry']['id'])
			{
				$app_dir = $content['path'] = Link::vfs_path($content['entry']['app'],$content['entry']['id'],'',true);
				if ($content['action'] == 'save_as') $path = $app_dir;
			}
			if (self::check_dir_writable($path, $msg))
			{
				error_log(array2string($content));
				$upload_success = $upload_failure = array();
				foreach($content['upload'] as $upload)
				{
					if ($upload )//&& Vfs::copy_uploaded($upload,$target=Vfs::concat($path,Vfs::encodePathComponent($upload['name'])),$content,false))
					{
						$target=Vfs::concat($path,Vfs::encodePathComponent($upload['name']));
						if (Vfs::file_exists($target) && $content['action'] == 'link_to' && $app_dir)
						{
							if (!Vfs::file_exists($app_dir)) Vfs::mkdir($app_dir);
							error_log("Symlinking $target to $app_dir");
							Vfs::symlink($target, Vfs::concat($app_dir,Vfs::encodePathComponent($upload['name'])));
						}
						$upload_success[] = $upload['name'];
					}
					else
					{
						$upload_failure[] = $upload['name'];
					}
				}
				$msg = '';
				if ($upload_success)
				{
					$msg = count($upload_success) == 1 && !$upload_failure ? lang('File successful uploaded.') :
						lang('%1 successful uploaded.',implode(', ',$upload_success));
				}
				if ($upload_failure)
				{
					$msg .= ($upload_success ? "\n" : '').lang('Error uploading file!');
				}
				if ($upload_success)
				{
					$enc_path = str_replace("'","\\'",$path);
					Framework::refresh_opener($msg,'filemanager',$enc_path,'edit',null,'/&path=[^&]*/',$enc_path);
				}
				if (!$upload_failure)
				{
					Framework::window_close();
					exit();
				}
			}
		}
		$content['msg'] = $msg;
		$content['help'] = lang('Stores files either in the entry directory, or in given path plus linking them to the entry directory.');

		$sel_options = array(
			'action' => array(
				'save_as' => lang('Save as'),
				'link_to' => lang('Link to'),
			),
		);

		$GLOBALS['egw_info']['flags']['app_header'] = lang('File a file');
		$tpl = new Etemplate('stylite.filemanager.upload');
		$tpl->exec('stylite.stylite_filemanager.upload',$content,$sel_options,array(),array(),2);
	}

	/**
	 * Check if given directory exists and is writable by current user
	 *
	 * @param string $path
	 * @param string &$msg error message on return, if there's an error
	 * @return boolean true if dir exists and is writable, false otherwise
	 */
	private static function check_dir_writable($path,&$msg)
	{
		if (!Vfs::file_exists($path) || !Vfs::is_dir($path))
		{
			$msg = lang('Directory not found or no permission to access it!');
			return false;
		}
		elseif (!Vfs::is_writable($path))
		{
			$msg = lang('Directory %1 is NOT writable by you!',$path);
			return false;
		}
		return true;
	}
}
