<?php
/**
 * EPL Function - Hooks
 *
 * @link www.egroupware.org
 * @author rb(at)egroupware.org
 * @copyright (c) 2009-16 by rb(at)egroupware.org
 * @package epl
 * @version $Id$
 */

use EGroupware\Api;
use EGroupware\Api\Egw;

class stylite_hooks
{
	const VERSION = "\105\107\162\157\165\160\167\141\162\145\134\123\164\171\154\151\164\145\134\126\146\163\134\126\145\162\163\151\157\156\151\156\147\134\123\164\162\145\141\155\127\162\141\160\160\145\162\72\72\164";
	/**
	 * Add search to topmenu
	 *
	 * @param string|array $data hook-data
	 */
	public static function topmenu_info($data)
	{
		unset($data);	// unused, but required by function signature

		if ($GLOBALS['egw_info']['user']['apps']['stylite'])
		{
			$GLOBALS['egw']->framework->add_topmenu_item(
				'search',
				Egw::link('/index.php', array('menuaction' => 'stylite.stylite_link_search_ui.search', 'ajax'=>'true'),'about'),
				lang('Search')
			);
		}
	}

	/**
	 * Replace api version with epl-version
	 *
	 * @param string|array $data hook-data
	 */
	public static function after_navbar($data)
	{
		unset($data);	// unused, but required by function signature

		$GLOBALS['egw_info']['server']['versions']['api'] =
			'EPL '.$GLOBALS['egw_info']['apps']['stylite']['version'].call_user_func(self::VERSION);
	}

 	/**
	 * Add further imap and smtp server plugins to emailadmin
 	 *
	 * @param string|array $data location string or array with key 'location' and other params
	 * @return array
 	 */
	public static function server_types($data)
 	{
		unset($data);	// unused, but required by function signature

 		// all moved to emailadmin, but we got a fatal error, if hooks got not registered again
 	}

	/**
	 * Hook to add reports sidebox
	 *
	 * @param array $data
	 */
	public static function report($data)
	{
		list(,$app) = explode('sidebox_', $data['location']);

		// Only show tracker reports to admins or technicians
		if($app == 'tracker')
		{
			$bo = new tracker_bo();
			if(!$bo->is_admin(0)) {
				return;
			}
		}
		$menu = array(
			'Reports' => Egw::link('/index.php', "menuaction=stylite.{$app}_report.index&ajax=true")
		);
		display_sidebox($app,lang('Reports'),$menu);
	}

	/**
	 * Admin hook
	 *
	 * @param string|array $data hook-data
	 */
	public static function admin($data)
	{
		$location = is_array($data) ? $data['location'] : $data;

		if ($GLOBALS['egw_info']['user']['apps']['admin'])
		{
			$file = Array(
				'PBX integration' => Egw::link('/index.php','menuaction=admin.admin_config.index&appname=stylite&ajax=true'),
				'View generator' => Egw::link('/index.php','menuaction=stylite.stylite_view.generator&ajax=true'),
				'License key' => Egw::link('/index.php','menuaction=stylite.stylite_license.enter&ajax=true'),
			);
			if ($location == 'admin')
			{
				display_section('stylite',$file);
			}
			else
			{
				display_sidebox('stylite',lang('Admin'),$file);
			}
		}
	}

	public static function groupdav_root_props(array $data)
	{
		$data['props'][] = Api\CalDAV::mkprop('http://stylite.de/', 'epl-key',
			sha1('dmfs4stylite'.preg_replace('/:\d+$/', '', $_SERVER['HTTP_HOST'])));
	}

	public static function pgp_encryption()
	{
		Api\Hooks::pgp_encryption_menu('infolog');
	}

	/**
	 * Enable Collabora configuration
	 *
	 * @param array $data content of request plus special keys "location_name" and "location_object" (=Etemplate object)
	 * @return array Modifications to make to the response.  These changes are
	 * made before any processing is done on the template exec().
	 *  String[] 'data':		Changes to the content
	 *  String[] 'readonlys':	Changes to the readonlys
	 *	String[] 'preserve':	Changes to preserve
	 */
	public static function etemplate2_before_exec(array $data)
	{
		if ($data['location_name'] !== 'admin.site-config' || $data['template'] !== 'collabora.config') return;

		return array(
			'data' => array(
				'epl' => true,
			)
		);
	}
}
