<?php
/**
 * EPL Functions: Reporting that uses jqPlot for charts
 *
 * @package epl
 * @author Nathan Gray
 * @copyright 2011 Nathan Gray
 * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
 * @version $Id$
 */

use EGroupware\Api;
use EGroupware\Api\Framework;
use EGroupware\Api\Etemplate;

abstract class stylite_jqplot_report extends stylite_report
{

	public $public_functions = array(
		'index' => true,
		'ajax_chart' =>true
	);

	/**
	 * Generate UI
	 */
	public static function index($content = array())
	{
		// Local source
		Framework::includeJS('/vendor/npm-asset/as-jqplot/dist/jquery.jqplot.js');
		// Bar
		Framework::includeJS('/vendor/npm-asset/as-jqplot/dist/plugins/jqplot.barRenderer.min.js');
		Framework::includeJS('/vendor/npm-asset/as-jqplot/dist/plugins/jqplot.categoryAxisRenderer.min.js');
		Framework::includeJS('/vendor/npm-asset/as-jqplot/dist/plugins/jqplot.canvasAxisLabelRenderer.min.js');
		Framework::includeJS('/vendor/npm-asset/as-jqplot/dist/plugins/jqplot.canvasTextRenderer.min.js');
		Framework::includeJS('/vendor/npm-asset/as-jqplot/dist/plugins/jqplot.highlighter.min.js');
		Framework::includeJS('/vendor/npm-asset/as-jqplot/dist/plugins/jqplot.pointLabels.min.js');

		// Pie
		Framework::includeJS('/vendor/npm-asset/as-jqplot/dist/plugins/jqplot.pieRenderer.min.js');
		Framework::includeJS('/vendor/npm-asset/as-jqplot/dist/plugins/jqplot.doughnutRenderer.min.js');

		// IE is 'special'
		if(Api\Header\UserAgent::type() == 'msie') Framework::includeJS('jquery/jqplot','excanvas.min');

		Framework::includeCSS('/vendor/npm-asset/as-jqplot/dist/jquery.jqplot.css');
/*
		Framework::includeCSS('/api/js/dojo/dojo/resources/dojo.css');
		Framework::includeCSS('/api/js/dojo/dijit/themes/claro/claro.css');
		Framework::includeCSS('/api/js/dojo/dojox/gantt/resources/gantt.css');
*/

		$tmpl = new Etemplate('stylite.report_jqplot');

		$app = $GLOBALS['egw_info']['flags']['currentapp'] ? $GLOBALS['egw_info']['flags']['currentapp'] : 'stylite';

		// Load app's lang
		Api\Translation::add_app($app);

		$name = $content['new_name'] ? $content['new_name'] : $content['report_name'];
		unset($content['new_name']);

		// Remember old values so we can tell if they changed
		$preserve['old_report'] = $name;
		$preserve['old_time_frame'] = $content['parameters']['time_frame'];

		// Make sure there's a group by
		if($content['parameters']['time_frame'] !== self::NOW && !$content['parameters']['group_by']) {
			$content['parameters']['group_by'] = 'cat_id';
		}

		// Store report settings
		if($name && ($content['save'] || $content['update'])) {
			self::save_report($name, $app, $content['parameters']);
		}

		// Update view of graph
		if($content['update']) {
			// Store it in the session for JpGraph
			unset($name);
			Api\Cache::setSession($app, 'report', $content['parameters']);
		} elseif($content['report_name'] && $content['report_name'] != $content['old_report']) {
			// Load selected report
			$content['parameters'] = self::load_report($content['report_name'], $app);
			$content['report_name'] = $name;
		} elseif($content['export']) {
			$content['parameters']['app'] = $app;
			self::export($content['parameters']);
		} elseif($content['delete'] && $content['report_name']) {
			self::delete_report($content['report_name'], $app);
			$content = array();
		}

		$data = $content;

		$static_class = call_user_func(array($app ? $app.'_report' : 'stylite_report', 'instance'),array());

		if(!$data['parameters']['time_frame']) $data['parameters']['time_frame'] = self::NOW;
		$sel_options = array(
			'time_frame' => self::$time_frame,
			'group_by' => $static_class->group_by_options,
			'report_name' => self::report_list($app),
		) + $static_class->select_options;
		$readonly = array();

		// Look for some custom fields that could work
		if(self::get_custom_fields($app, $data, $sel_options,
			$static_class->type_field && $data['parameters'][$static_class->type_field] ? $data['parameters'][$static_class->type_field] : null))
		{
			$tmpl->disable_cells('cf');
		}

		// Disable exporting?
		$config = Api\Config::read('phpgwapi');
		$limit_exception = count(array_intersect(array($GLOBALS['egw_info']['user']['account_id']) + $GLOBALS['egw']->accounts->memberships($GLOBALS['egw_info']['user']['account_id'],true), (array)unserialize($GLOBALS['egw_info']['server']['export_limit_excepted']))) > 0;
		if($config['export_limit'] == 'no' && !($GLOBALS['egw_info']['account']['apps']['admin'] || $limit_exception)) {
			$tmpl->disable_cells('export');
		}
		$data['parameters']['params_allowed'] = true;

		// Allow extending app a chance
		if($app != 'stylite') {
			if(!is_array($content['parameters'])) $content['parameters'] = array();
			if(!is_array($data['parameters'])) $data['parameters'] = array();

			$class = "{$app}_report";
			call_user_func_array(array($class, 'get_parameters'), array(&$content['parameters'], &$data['parameters'], &$sel_options, &$readonly));
		}
		$data['parameters']['no_group_by'] = $data['parameters']['time_frame'] == self::NOW;
		$data['report'] = Api\Html::image('stylite', array('menuaction' => 'stylite.'.$app . '_report.image','app' => $app, 'report' => $name), $name);
		$data['parameters']['report_parameters'] = $app !== 'stylite' ? "stylite.$app.report" : ''; // Don't use stylite, causes a loop
		$data['parameters']['app'] = $app;

		Framework::includeJS('report','jqplot','stylite');

		return $tmpl->exec("stylite.{$app}_report.index", $data, $sel_options, $readonly, $preserve);

	}

	public static function ajax_chart($parameters) {

		// Ajax submit has the whole POST, not just form variables
		if(is_array($parameters['parameters']))
		{
			$parameters = $parameters['parameters'];
		}

		$name = trim($parameters['report']); // Only here if explicitly sent, regular POST used to load
		$app = $parameters['app'];
		if($name) {
			$parameters = self::load_report($name);
		} elseif($app) {
			$name = lang($app);
		} elseif (!is_array($parameters)) {
			return;
		}
		if($parameters['params']) $parameters = unserialize($parameters['params']);
		if(!$parameters['app']) $parameters['app'] = $app;

		Api\Translation::add_app($parameters['app']);

		// Needed until static:: becomes available
		$static_class = call_user_func(array($app . '_report', 'instance'),array());

		//_debug_array($parameters);die();

		// Get start / end / resolution from timeframe
		$static_class->get_timeframe($parameters);

		$response = Api\Json\Response::get();

		if($parameters['time_frame'] == self::NOW) {
			// Current values
			try {
				// No group by for current view
				unset($parameters['group_by']);
				$data = $static_class->get_current($parameters);
				$graph = $static_class->get_current_graph($parameters, $data);
			} catch (Exception $e) {
				$response->error( $e->getMessage());
			}
		}
		else
		{
			// Historical
			try {
				$data = $static_class->get_aged($parameters);
				if(!$data) {
					$data = array(''=>array(0));
				}
				$graph = $static_class->get_history_graph($parameters, $data);
				$graph[0]['title'] = $name .': ' . lang($static_class->group_by_options[$parameters['group_by']]);
			} catch (Exception $e) {
				$response->error( $e->getMessage());
			}

		}

		$response->data($graph);

		return $graph;
	}

	/**
	 * Set graph subtitle based on filter parameters
	 */
	protected static function set_subtitle(&$graph, $parameters) {
		$subtitle = '';
		$subsubtitle = '';

		if($parameters['time_frame'] == self::NOW) {
			$subsubtitle = Api\DateTime::to();
		} else {
			$subsubtitle = Api\DateTime::to($parameters['start'],true) . ' - ' . Api\DateTime::to($parameters['end'],true);
		}
		$graph['subtitle'] = $subtitle;
		$graph['subsubtitle'] = $subsubtitle;
	}

	/**
	 * Build the graph showing the current snapshot
	 * Graph is a series of pie charts showing current breakdown, one pie per sub-array in data
	 *
	 * Returned data = array(
	 *	one per pie (
	 *		type	=>	'Pie',
	 *		title	=>	Pie title,
	 *		series	=>	array(
	 *			array(
	 *				legend	=> displayed in legend,
	 *				tooltip	=> Displayed on slice mouseover,
	 *				value	=> Size of slice
	 *			)
	 *			...
	 *		)
	 *	)
	 *
	 * @param parameters Search / filter parameters limiting results
	 * @param data Array of data used to make multiple pie graphs
	 *
	 * @return array Data for graph, in a format suitable for display.
	 */
	protected static function get_current_graph(Array $parameters, Array $data)
	{
		unset($parameters);	// not used, but required by function signature

		$graph = array();

		// How many slices
		$limit = 10;
		foreach($data as $series_name => $values)
		{
			$series = array();

			if(count($values) > $limit )
			{
                                $total = array_sum($values);
                                asort($values);
                                $values = array_reverse($values,true);
                                $legend_index = 1;
                                foreach($values as $name => $count) {
                                        if($count < $total * .07 || $legend_index++ >= $limit) {
                                                // Add [] around Other in case app already has an 'Other'
                                                $values['['.lang('Other').']'] += $count;
                                                unset($values[$name]);
                                        }
                                }
                        }

			foreach($values as $name => $value)
			{
				$series[] = array(
					'value'		=> $value,
					'legend'	=> $name,
					'tooltip'	=> sprintf('%s: %d',$name, $value)
				);
			}
			$graph[] = array(
				'type'	=> 'Pie',
				'title'	=> $series_name,
				'series'	=> $series
			);
		}
		return $graph;
	}

	/**
	 * Build the graph showing the historical changes
	 * Graph is a stacked bar graph, showing count changes over time
	 *
	 * Graph data is massaged a little more on client side, but:
	 * data = array(
	 *	type		=> 'StackedColumns' // Type for jqPlot graph
	 *	x_label		=> Label for x axis
	 *	y_label		=> Label for y axis
	 *	x_labels	=>	Lables for ticks (columns) in x axis
	 *	title		=> Main title for graph
	 *	subtitle	=>	Filters applied
	 *	subsubtitle	=>	Date range
	 *	series	=> array of data series (
	 *		array(
	 *			legend	=> ,
	 *			tooltip	=> Shown on mousover,
	 *			values	=> array(0,1,2...)
	 *		)
	 *	)
	 *
	 * @param parameters Search / filter parameters limiting results
	 * @param data Array of data used to make a stacked bar graph
	 *
	 * @return array Data for graph, in a format suitable for display by jqPlot.
	 */
	protected static function get_history_graph(Array $parameters, Array $data)
	{
		$static_class = call_user_func(array($parameters['app'] . '_report', 'instance'),array());

		// Limit series count to 20
		$static_class->limit_series($data, 20);

		// Sort values for consistancy
		ksort($data);

		$plots = array();
		foreach($data as $series => $series_data) {
			$series_plot = array();
			$data = array();

			// Legend
			$series_total = array_sum($series_data);
			if($static_class->select_options[$parameters['group_by']][$series]) {
				$series = ucfirst($static_class->select_options[$parameters['group_by']][$series]);
			} elseif(is_numeric($series) && $series) {
				// Label has since been deleted
				$series = lang('Deleted') .' '. $parameters['group_by'] . '-'.$series;
			} else {
				// Series is already labeled
				$series = ucfirst($series);
			}
			$series_plot['legend'] = ucfirst($series) . " ($series_total)";

			// Total by date group, make tooltips
			foreach($series_data as $date => $value) {
				$date_totals[$date] += $value;
				if($parameters['time_slice'] == self::WEEK) {
					list($year,$week) = explode('-',$date);
					$date = self::monday_week_n($week, $year, 'Y-m-d'); // Use Y-m-d for consistancy with monthly
				}
				$series_plot['tooltip'][] = "$series<br />$date: $value";
			}

			// Add data
			$series_plot['values'] = array_values($series_data);

			$plots[] = $series_plot;
		}

		$x_labels = array();
		foreach($date_totals as $date => $date_total) {
			// Format dates (weeks)
			if($parameters['time_slice'] == self::WEEK) {
				list($year,$week) = explode('-',$date);
				$date = self::monday_week_n($week, $year, 'Y-m-d'); // Use Y-m-d for consistancy with monthly
			}
			$x_labels[] = sprintf('%s (%d)', $date, $date_total);
		}
		$x_label = $static_class->field2label[$static_class->date_field] ? $static_class->field2label[$static_class->date_field] : lang('Created');
		$graph = array(
			'type'	=>	'StackedColumns',
			'x_label'	=> $x_label . ' (' .lang('Total').")<br />\n".lang('Total').': '.array_sum($date_totals),
			'x_labels'	=> $x_labels,
			'y_label'	=> '# of entries',
			'series'	=> $plots
		);

		// $name is undefined: $graph['title'] = $name;
		// Sub-title
		$static_class->set_subtitle($graph, $parameters);

		return array($graph);
	}
}
