<?php
/**
 * EPL Functions - Enter EPL license key
 *
 * @link www.egroupware.org
 * @author rb(at)egroupware.org
 * @copyright (c) 2011-16 by rb(at)egroupware.org
 * @package epl
 * @version $Id$
 */

use EGroupware\Api;
use EGroupware\Api\Framework;
use EGroupware\Api\Egw;
use EGroupware\Api\Etemplate;

/**
 * Enter EPL license key
 */
class stylite_license
{
	/**
	 * Methods callable via menuaction
	 */
	public $public_functions = array(
		'enter' => true,
	);

	/**
	 * Enter EPL license key
	 *
	 * @param array $content=null
	 * @param string $msg=''
	 */
	function enter(array $content=null, $msg='', $msg_type='success')
	{
		if (!is_array($content))
		{
			$content['num_users'] = stylite_license_bo::active_users();

			$content['install_id'] = $GLOBALS['egw_info']['server']['install_id'];
			$content['licensekey'] = $GLOBALS['egw_info']['server']['epllicensekey'];
			$content['my-stylite-username'] = $GLOBALS['egw_info']['server']['my-stylite-username'];
			$content['my-stylite-password'] = $GLOBALS['egw_info']['server']['my-stylite-password'];

			if (empty($content['licensekey']) && !stylite_license_bo::ucs_csp_license())
			{
				$msg = lang('Enter either license key or my.egroupwar.org username and password to request license!');
				$msg_type = 'info';
			}
			if (md5_file(__DIR__.'/class.stylite_license_bo.inc.php') != 'fbf03f2c9e78e56376145ceabec3f9db' ||
				stylite_license_bo::validate_hooks(false))
			{
				$msg = lang('License logic modified!');
				$msg_type = 'error';
			}
		}
		elseif($content['save'])
		{
			unset($content['save']);
			if (!trim($content['licensekey']) && !stylite_license_bo::ucs_csp_license())
			{
				$required_missing = array();
				foreach(array('my-stylite-username', 'my-stylite-password') as $field)
				{
					if (empty($content[$field]))
					{
						Etemplate::set_validation_error($field, lang('Field must not be empty !!!'), '');
						$required_missing[] = $field;
					}
					else
					{
						Api\Config::save_value($field, $content[$field], 'phpgwapi');
					}
				}
				if (!$required_missing)
				{
					try {
						$content['licensekey'] = stylite_license_bo::request_license($content['my-stylite-username'], $content['my-stylite-password']);
					}
					catch (Exception $ex)
					{
						$msg = $ex->getMessage();
						$msg_type = 'error';
					}
				}
			}
			if (trim($content['licensekey']))
			{
				Api\Config::save_value('epllicensekey', $content['licensekey'], 'phpgwapi');
				$msg = lang('License key saved.');
			}
		}
		elseif($content['cancel'])
		{
			Egw::redirect_link('/index.php', array(
				'menuaction' => 'admin.admin_ui.index',
				'ajax' => 'true'
			), 'admin');
		}
		if (($content['license_data'] = trim($content['licensekey']) ?
			stylite_license_bo::license_data($content['licensekey']) : stylite_license_bo::ucs_csp_license()))
		{
			try {
				if (!$content['license_data']['lic_users'])
				{
					$content['license_data']['lic_users'] = lang('unlimited');
				}
				// validate license-data
				if (($content['license_data']['error'] = stylite_license_bo::validate($content['licensekey'], 'array')) && $msg_type != 'error')
				{
					$glue = count($content['license_data']['error']) > 1 ? "\n- " : '';
					$msg = lang('Invalid license').': '.$glue.implode($glue, $content['license_data']['error']);
					$msg_type = 'error';
				}
			}
			catch (Exception $ex) {
				$content['license_data'] = array();
				$msg = lang('Invalid license').': '.$ex->getMessage();
				$msg_type = 'error';
			}
		}
		Framework::message($msg, $msg_type);

		$tpl = new Etemplate('stylite.license');
		$tpl->exec('stylite.stylite_license.enter', $content, array(
			'lic_concurrent' => array(
				''  => 'active',
				'1' => 'concurrent',
			),
		), array(), $content);
	}
}
