<?php
/**
 * EPL Functions - Enter EPL license key
 *
 * @link www.egroupware.org
 * @author rb(at)egroupware.org
 * @copyright (c) 2011-16 by rb(at)egroupware.org
 * @package epl
 * @version $Id$
 */

use EGroupware\Api;
use EGroupware\Api\Framework;

/**
 * EPL license management
 *
 ***************************************************************
 * If you modify this file, your EGroupware will stop working! *
 ***************************************************************
 */
class stylite_license_bo
{
	const PUBKEY = '-----BEGIN PUBLIC KEY-----
MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDm2DGTq7gjwG4mc2AgS3jnyjPn
EBA0PAO1kjj9HaVUjoGbXjjeAPYmg2NCc8Mc/VGOE8Dta6i+pv3xXJloZev0+iUE
r4Ht6/vwnHctqSMEl4p85/6cmAdQBlgfk/t14+MxZ5QTe9HfqNl/TKLTmroOJ6Jf
IU5yeA6gHX3zgZyy6wIDAQAB
-----END PUBLIC KEY-----';

	/**
	 * Fields in colon separated license
	 * @var array
	 */
	private static $license_fields = array(
		'lic_id', 'lic_owner', 'account_email',
		'lic_version', 'lic_expires', 'lic_users', 'lic_install_id',
		'date', 'lic_concurrent',
	);

	/**
	 * Get data of license
	 *
	 * @param string $crypted64 base64 encoded license key
	 */
	public static function license_data($crypted64)
	{
		$crypted = base64_decode(str_replace("\r\n",'',$crypted64));
		$decrypted = null;
		if (!openssl_public_decrypt($crypted, $decrypted, self::PUBKEY))
		{
			throw new Exception('Failed to decrypt with public key!');
		}
		$data = explode(';', $decrypted);

		if (count($data) != count(self::$license_fields))
		{
			if (count($data) == count(self::$license_fields)-1)
			{
				$data[] = '';	// old license without concurrent flag, just add it
			}
			else
			{
				throw new Exception('Invalid number of fields!');
			}
		}

		return array_combine(self::$license_fields, $data);
	}

	private static $ucs_master_keys = array(
		'_MASTER-KEY' => '-----BEGIN PUBLIC KEY-----
MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA0jQVFvjqhr2mEUPsG5g2
kQA58uq7QMb0gFYOfhAQsQgMuhp2sjXsdkLG2QSLKhQf9RDZBbgZBffvU3DvRvaf
WVdX+iAR2AhYGy6pE5Mrj+iXgVcFrlxMDpVK5PF1N4iGwQpkMS6dgjgfVl+0b4kr
99BCU+bZoc/t/KlmGoXVrfPNEZMKa2fQbsHkPxTtGq2ylLP2JvGwlEOeUvsbm5H0
iOUzDwl35fQYK3um19VKxCIMLvtV95fJZvoZYJYb3sbI0bq3pJxUi/nLi0p1xGYz
xSA5nUf5FK53qFN+w9j6OA37URXo19YjUi0mrXNTr7ZiehGobPvKHBdBBtl7LuYL
hQIDAQAB
-----END PUBLIC KEY-----',
    '_MASTER-KEY2' => '-----BEGIN PUBLIC KEY-----
MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA06fyc7AmDJg3nzCEB4vP
HDBhkTJcMof5fdhWsp049JgQxCcXnbkFo10RBHT9TxlMjN4ZJ38QkMwh5E0wTc2A
/CRqJkVjghTUllPY/MqciftcSyDI0bEfQEi9rUluomMO615+spmLOWBGcnYH3JJU
kHwFOF/TYYkqZeFVbBqVtiGBOUXlSWbGBGAGVR15TfEuEUt0txjfQReIb+/d7/ei
AbX/rgiaq0E1iHOT3Lbqi+sUId31ti6G3WmuNVln+b5k0YruC9T5IIoOud/lz6A8
XaaAIS3eujulP79Xmw6yP+KVIHSFz2KRVGvSnWgKOyhuFR9/3hAyTeaSGFwRplEN
CQIDAQAB
-----END PUBLIC KEY-----',
);

	/**
	 * Verify UCS License via its signature
	 *
	 * @param array $license as returnd by ldap
	 */
	public static function verify_ucs_license(array $license)
	{
		if (substr($license['dn'], -strlen($license['univentionlicensebasedn'][0])-1) != ','.$license['univentionlicensebasedn'][0])
		{
			throw new Exception('UCS license is for different base DN!');
		}
		ksort($license);
		$data = '';
		foreach($license as $attr => $value)
		{
			if (substr($attr, 0, 17) == 'univentionlicense' && $attr != 'univentionlicensesignature')
			{
				$data .= $value[0]."\n";
			}
		}

		foreach(self::$ucs_master_keys as $pubkey)
		{
			if (openssl_verify($data, base64_decode($license['univentionlicensesignature'][0]), $pubkey))
			{
				return;
			}
		}
		throw new Execption ('Failed to verify UCS license signature!');
	}

	/**
	 * Get data of UCS CSP license
	 *
	 * @return array
	 */
	public static function ucs_csp_license()
	{
		if ($GLOBALS['egw_info']['server']['account_repository'] != 'univention' ||
			!is_a($GLOBALS['egw']->accounts->backend, 'accounts_univention'))
		{
			return null;
		}
		$parts = explode(',', $GLOBALS['egw']->accounts->backend->user_context);
		array_shift($parts);
		$context = implode(',', $parts);
		$ds = $GLOBALS['egw']->accounts->backend->ds;
		if (($sri = ldap_search($ds, $context, '(&(objectClass=univentionLicense)(univentionLicenseOEMProduct=CSP))')) &&
			($entries = ldap_get_entries($ds, $sri)) && $entries['count'] && ($data = $entries[0]))
		{
			self::verify_ucs_license($data);

			return array(
				'lic_id' => $data['univentionlicensetype'][0].'-'.$data['univentionlicenseoemproduct'][0].': '.$data['univentionlicensekeyid'][0],
				'lic_expires' => implode('-', array_reverse(explode('.', $data['univentionlicenseenddate'][0]))),
				'lic_users' => $data['univentionlicenseusers'][0] != 'unlimited' ? $data['univentionlicenseusers'][0] : '',
				'lic_concurrent' => '',	// active users
				'lic_install_id' => $GLOBALS['egw_info']['server']['install_id'],
				'lic_version' => preg_replace('/^(\d+\.\d+).*$/', '\\1', $GLOBALS['egw_info']['apps']['stylite']['version']),
			);
		}
		return null;
	}

	/**
	 * Validate license
	 *
	 * @param string $key =null
	 * @param string $return ='redirect' 'redirect' to license entry, 'throw' exception or return 'string' or 'array'
	 * @param int $offset =0
	 * @param boolean $exclude_current_user =false should current user be excluded from concurrent users
	 * @return string|array with validation messages for diverse license-fields or null
	 */
	public static function validate($key=null, $return='redirect', $offset=0, $exclude_current_user=false)
	{
		if (empty($key)) $key = $GLOBALS['egw_info']['server']['epllicensekey'];
		$error = null;

		Api\Translation::add_app('stylite');

		try {
			if (!trim($key))
			{
				if (!($data = self::ucs_csp_license()))
				{
					throw new Exception (lang('Missing license key!'));
				}
			}
			else
			{
				$data = self::license_data($key);
			}
			if ($data['lic_install_id'] && $data['lic_install_id'] != $GLOBALS['egw_info']['server']['install_id'])
			{
				$error['lic_install_id'] = lang('License is for different install-ID!');
			}
			if (!$data['lic_users'])	// unlimited users
			{

			}
			elseif (!$data['lic_concurrent'] && $data['lic_users'] < ($au=self::active_users())+$offset)
			{
				$error['lic_users'] = lang('License allows only %1 of your %2 active users!', $data['lic_users'], $au);
			}
			elseif ($data['lic_users'] < ($cu=self::concurrent_users($exclude_current_user))+(int)$exclude_current_user)
			{
				$error['lic_users'] = lang('License allows only %1 of your %2 concurrent users!', $data['lic_users'], $cu);
			}
			if ($data['lic_version'] && !version_compare($GLOBALS['egw_info']['apps']['stylite']['version'], $data['lic_version'], '<='))
			{
				$error['lic_version'] = lang('License is for version %1, you use %2!', $data['lic_version'], $GLOBALS['egw_info']['apps']['stylite']['version']);
			}
			if ($data['lic_expires'] && Api\DateTime::to($data['lic_expires'], 'ts') < time())
			{
				$error['lic_expires'] = lang('License is only valid until %1!', $data['lic_expires']);
			}
		}
		catch (Exception $ex) {
			$error['license_key'] = $ex->getMessage();
		}
		if ($error && $return === 'redirect')
		{
			Framework::redirect_link('/index.php', 'menuaction=stylite.stylite_license.enter', 'admin');
		}
		if ($error && $return !== 'array')
		{
			$glue = count($error) > 1 ? "\n- " : '';
			$error = lang('Invalid license').': '.$glue.implode($glue, $error);

			if ($return == 'throw' && !isset($GLOBALS['egw_setup']))
			{
				throw new Api\Exception\NoPermission($error);
			}
		}
		//error_log(__METHOD__."('$key') returning ".array2string($error));
		return $error;
	}

	public static function validate_hooks()
	{
		$info = self::hooks();
		foreach($info['hooks'] as $location => $method)
		{
			if (Api\Hooks::exists($location, 'stylite', true) === $method)
			{
				return dirname(__DIR__).'/setup/setup.inc.php';
			}
		}
	}

	/**
	 * Get number of active users
	 *
	 * @return int
	 */
	public static function active_users()
	{
		$ab = $GLOBALS['egw']->contacts;
		$hide_accounts = $GLOBALS['egw_info']['user']['preferences']['addressbook']['hide_accounts'];
		$GLOBALS['egw_info']['user']['preferences']['addressbook']['hide_accounts'] = '0';
		$ab->search('',true,'','','','','',array(0,1),array('owner' => 0));
		$GLOBALS['egw_info']['user']['preferences']['addressbook']['hide_accounts'] = $hide_accounts;
		return $ab->total;
	}

	public static function concurrent_users($exclude_current_user=false)
	{
		$filter = array();
		$filter['lo'] = null;
		$filter[] = 'account_id > 0';
		$filter[] = 'session_dla > '.(int)(time() - $GLOBALS['egw_info']['server']['sessions_timeout']);
		$filter[] = '(notification_heartbeat IS NULL OR notification_heartbeat > '.Api\Session::heartbeat_limit().')';
		if ($exclude_current_user) $filter[] = 'account_id != '.(int)$GLOBALS['egw_info']['user']['account_id'];
		$ret = $GLOBALS['egw']->db->select(Api\Session::ACCESS_LOG_TABLE, 'COUNT(DISTINCT account_id)', $filter, __LINE__, __FILE__)->fetchColumn();
		//error_log(__METHOD__."($exclude_current_user) egw_info[user][account_id]={$GLOBALS['egw_info']['user']['account_id']} returning $ret");
		return $ret;
	}

	const MY_STYLITE_URL = 'https://my.egroupware.org/egw/licenses/license.php';
	const INSTALL_ID_PREG = '/^[a-f0-9]{32}$/i';

	/**
	 * Request license-key from my.egroupware.org using install_id and my-stylite-(username|password)
	 *
	 * @param string $username
	 * @param string $password
	 * @return string license-key
	 */
	public static function request_license($username, $password)
	{
		$install_id = $GLOBALS['egw_info']['server']['install_id'];
		// create valid install_id, if there's none
		if (!preg_match(self::INSTALL_ID_PREG, $install_id))
		{
			$install_id = md5($_SERVER['HTTP_HOST'].microtime(true));
			Api\Config::save_value('install_id', $install_id, 'phpgwapi');
		}
		$url = self::MY_STYLITE_URL.'?install_id='.$install_id;

		if (($data = file_get_contents($url, false, Framework::proxy_context($username, $password))) === false)
		{
			if (!ini_get('allow_url_fopen'))
			{
				throw new Api\Exception\NoPermission(lang('%1 setting "%2" = %3 disallows access via http!',
					'php.ini', 'allow_url_fopen', array2string(ini_get('allow_url_fopen'))));
			}
			$matches = null;
			if (preg_match('|^HTTP/[0-9.]+ ([0-9]+)|', $http_response_header[0], $matches) && $matches[1] == 401)
			{
				throw new Api\Exception\NoPermission(lang('Wrong username or password!'));
			}
			throw new Api\Exception\NoPermission('Webserver returned: '.$http_response_header[0]);
		}
		$data = json_decode($data, true);
		if (!$data || !is_array($data))
		{
			throw new Api\Exception('Invalid data returned!');
		}
		if (isset($data['Exception']))
		{
			$class = $data['type'];
			if (!class_exists($class) || !is_a($class, 'Exception')) $class = 'Exception';
			throw new $class($data['message'], $data['code']);
		}
		return $data['license'];
	}

	public static function pre_account($data)
	{
		if ($data['location'] == 'pre_addaccount' || $data['account_status'])
		{
			self::validate(null, 'throw', (int)($data['location'] == 'pre_addaccount' ||
				$data['account_id'] && !$GLOBALS['egw']->accounts->id2name($data['account_id'], 'account_status')));
		}
	}

	public static function addressbook_edit(array $content)
	{
		if ((string)$content['owner'] === '0' && $GLOBALS['egw_info']['user']['apps']['admin'])
		{
			if (($error = self::validate(null, 'string', (int)!$content['account_id'])))
			{
				Framework::message($error, 'error');
			}
		}
	}

	public static function session_creation(array $data)
	{
		unset($data);

		if (!isset($GLOBALS['egw_info']['user']['apps']['admin']))
		{
			return self::validate(null, 'string', 0, true);
		}
		return false;
	}

	public static function hooks()
	{
		return array(
			__FUNCTION__ => array(
				'pre_addaccount' => __CLASS__.'::pre_account',
				'pre_editaccount' => __CLASS__.'::pre_account',
				'addressbook_edit' => __CLASS__.'::addressbook_edit',
				'session_creation' => __CLASS__.'::session_creation',
		));
	}
}