<?php
/**
 * EPL Functions - search
 *
 * Search though all applications using the linking system, display all matching results
 * up to RESULT_COUNT for each application.  If there are more results, and the application supports the view_list
 * hook, a 'More...' link will be displayed to take the user to the full list of results for that app.
 *
 * @author Nathan Gray
 * @package epl
 * @copyright (c) 2003-16 by Nathan Gray <NathanGray@users.sourceforge.net>
 */

use EGroupware\Api;
use EGroupware\Api\Link;
use EGroupware\Api\Framework;
use EGroupware\Api\Etemplate;

class stylite_link_search_ui {

	const RESULT_COUNT = 10;

	public $public_functions = array(
		'search'	=>	true,
		'ajax_query' => true
	);

	protected static $template;

	public function __construct() {
	}

	public function search($content = array()) {
		$data = array();
		$apps = Link::app_list('query');
		self::$template = new Etemplate('stylite.link_search.search');

		// Skip these
		unset($apps['addressbook-email']);

		// Only include Api\Accounts if account storage is different from contact storage
		$addressbook = new Api\Contacts\Storage();
		if($addressbook->account_repository == $addressbook->contact_repository)
		{
			unset($apps['api-accounts']);
		}
		if ($apps)
		{
			asort($apps);   // sort them alphabetic

			foreach(array_keys($apps) as $app) {
				$app_result = array(
					'app'		=>	$app,
					'appname'	=>	lang($app),
					'icon'		=>	$app . '/navbar',
					'disabled'	=> true
				);
				if(Link::$app_register[$app]['view_list']) {
					$app_result['list'] = Link::$app_register[$app]['view_list'];
				}

				// Sub-types
				if(is_array(Link::$app_register[$app]['types'])) {
					foreach(Link::$app_register[$app]['types'] as $key => $type) {
						$app_result['type'] = $key;
						$app_result['appname'] = lang($app) . ' - ' . lang($type['name']);
						$app_result['icon'] = $type['icon'] ? $type['icon'] : $app_result['icon'];

						$data['apps'][] = $app_result;
					}
				} else {
					$data['apps'][] = $app_result;
				}
				self::$template->disableElement("apps[$app]",true);
			}
		}

		$data['query'] = $content['query'];

		// JavaScript
		Framework::includeJS('link_search', 'link_search', 'stylite');

		$GLOBALS['egw_info']['flags']['app_header'] = lang('Site search');
		self::$template->exec('stylite.stylite_link_search_ui.search', $data);
	}

	/**
	*	Query to get the results.  Called once for each application.
	*
	*	@param String etemplate_exec_id
	*	@param String[] app => div_id Name of the application, and it's div ID
	*	@param pattern Search for this text
	*	@param options Options to pass along for the query such as result limits
	*
	*	@return AJAX XML response
	*/
	public static function ajax_query($etemplate_exec_id, $div_id, $app, $pattern, $options)
	{
		unset($div_id);	// not used

		$response = Api\Json\Response::get();

		$result = stylite_link_search_ui::query($app, $pattern, $options, $etemplate_exec_id);
		$response->data($result);
		$response->call('link_search.show_more', $app, ($options['total'] > $options['num_rows']));
	}

	public static function query($app, $pattern, &$options = array(), $etemplate_request_id = '')
	{
		unset($etemplate_request_id);	// not used

		$app_result = array();
		$options['start']	=	0;
		$options['num_rows']	=	self::RESULT_COUNT;

		try {
			// Make sure any extra output doesn't spoil the results
			ob_start();
			$results = Link::query($app, $pattern, $options);
			if(is_array($results) && count($results) > 0) {
				foreach($results as $id => $title) {
					$new_line = array(
						'app'	=>	$app,
						'id'	=>	$id,
					);
					if(is_array($title)) {
						$new_line['title'] = $title['label'];
						$new_line = array_merge($new_line, $title);
					} else {
						$new_line['title'] = $title;
					}
					$app_result[] = $new_line;
				}

			}

			// Send any extra output to the error log
			if(ob_get_length() > 0) {
				error_log("Extra output searching $app: " . ob_get_contents());
			}
			ob_end_clean();
		} catch (Exception $e) {
			error_log($e->getMessage());
		}
		return $app_result;
	}
}
