<?php
/**
 * EPL Functions - EPL view generator
 *
 * @link www.egroupware.org
 * @author rb(at)egroupware.org
 * @copyright (c) 2012-16 by rb(at)egroupware.org
 * @package epl
 * @version $Id$
 */

use EGroupware\Api;
use EGroupware\Api\Etemplate;

/**
 * EPL view generator
 *
 * Views are stored in EGroupware configuration for app 'stylite' and name 'view-'<name of view>.
 */
class stylite_view
{
	/**
	 * Methods callable via menuaction
	 */
	public $public_functions = array(
		'generator' => true,
	);

	const APP_NAME = 'stylite';

	/**
	 * Informaton about supported apps
	 *
	 * @var array of appname => data pairs, data supports keys 'main', 'extra'
	 */
	public static $apps = array(
		'addressbook' => array(
			'main' => 'egw_addressbook',
			'extra' => 'egw_addressbook_extra',
			'id_col' => 'contact_id',
			'extra_name_col' => 'contact_name',
			'extra_value_col' => 'contact_value',
			'app' => 'phpgwapi',
			'convert' => array(
				'contact_owner' => 'account',	// is for some reason int8
			),
		),
		'calendar' => array(
			'main' => 'egw_cal',
			'extra' => 'egw_cal_extra',
			'id_col' => 'cal_id',
			'extra_name_col' => 'cal_extra_name',
			'extra_value_col' => 'cal_extra_value',
		),
		'infolog' => array(
			'main' => 'egw_infolog',
			'extra' => 'egw_infolog_extra',
			'id_col' => 'info_id',
			'extra_name_col' => 'info_extra_name',
			'extra_value_col' => 'info_extra_value',
		),
		'timesheet' => array(
			'main' => 'egw_timesheet',
			'extra' => 'egw_timesheet_extra',
			'id_col' => 'ts_id',
			'extra_name_col' => 'ts_extra_name',
			'extra_value_col' => 'ts_extra_value',
			'convert' => array(
				'ts_duration' => '',	// no conversation for ts_duration, it's no timestamp
			),
		),
		'tracker' => array(
			'main' => 'egw_tracker',
			'extra' => 'egw_tracker_extra',
			'id_col' => 'tr_id',
			'extra_name_col' => 'tr_extra_name',
			'extra_value_col' => 'tr_extra_value',
		),
		'projectmanager' => array(
			'main' => 'egw_pm_projects',
			'extra' => 'egw_pm_extra',
			'id_col' => 'pm_id',
			'extra_name_col' => 'pm_extra_name',
			'extra_value_col' => 'pm_extra_value',
		),
	);

	/**
	 * Available conversations
	 *
	 * @var array
	 */
	public static $conversations = array(
		'cat' => 'Category name',
		'cat2' => '2 category names',
		'cat3' => '3 category names',
		'cat5' => '5 category names',
		'account' => 'Account name',
		'account2' => '2 account names',
		'account3' => '3 account names',
		'account5' => '5 account names',
		'timestamp' => 'Timestamp',
		'cf-label' => 'CF value to label',
	);

	/**
	 * View generator UI
	 *
	 * @param array $content =null
	 * @param string $msg =''
	 */
	public function generator(array $content=null, $msg='')
	{
		$config = Api\Config::read(self::APP_NAME);
		// migrate existing old array of all views (named "views") as single views (named "view-$name")
		$views = (array)$config['views'];
		if ($views)
		{
			foreach($views as $name => $view)
			{
				Api\Config::save_value('view-'.$name, $view, self::APP_NAME);
			}
			Api\Config::save_value('views', null, self::APP_NAME);
		}
		// load new views named "view-$name"
		foreach($config as $name => $view)
		{
			if (substr($name, 0, 5) == 'view-')
			{
				$views[substr($name, 5)] = $view;
			}
		}

		if (is_array($content))
		{
			list($app) = explode(':', $content['view']);
			if (isset($content['save']) || isset($content['create']) || isset($content['delete']))
			{
				if ($content['delete'])
				{
					unset($views[$content['old_view']]);
					Api\Config::save_value('view-'.$content['old_view'], null, self::APP_NAME);
					$msg = lang('View definition deleted.');
					$content['view'] = $app;
				}
				else
				{
					$content['view'] = $app.':'.$content['name'];
					// index defintion with column-name, to allow merging it with default view, to get new columns
					$views[$content['view']] = array();
					foreach($content['defs'] as $def)
					{
						if ($def) $views[$content['view']][$def['column']] = $def;
					}
					Api\Config::save_value('view-'.$content['view'], $views[$content['view']], self::APP_NAME);
					$msg = lang('View definition saved.');
				}
				unset($content['delete']);
				unset($content['save']);
			}
			if (isset($content['create']))
			{
				unset($content['create']);
				try {
					$GLOBALS['egw']->db->query($sql = self::sql_view($content['name'], $content['defs'], $app));

					$msg .= "\n".lang('View successful created or updated on database.');
				}
				catch (Api\Db\Exception $e) {
					$msg = lang('Error creating view').': '.$e->getMessage();
				}
			}
		}
		else
		{
			$content['view'] = $app = 'addressbook';
		}
		$app_data = self::$apps[$app];
		if (isset($app_data) && ($content['view'] !== $content['old_view'] || !$content['defs']))
		{
			if ($content['view'] == $app || !isset($views[$content['view']]))	// default view of app
			{
				$content['defs'] = $this->default_view($app);
				$content['name'] = $app;
			}
			else
			{
				// merge with default view, to get evtl. existing new columns or cf's (disabled)
				$content['defs'] = array_merge($this->default_view($app, false), $views[$content['view']]);
				list(,$content['name']) = explode(':', $content['view']);
			}
			// renumber starting with 1 (0 is header row)
			$content['defs'] = array_values($content['defs']);
			array_unshift($content['defs'], false);
		}
		$content['sql'] = $sql;
		$content['msg'] = $msg;
		//_debug_array($content);
		$sel_options = array(
			'view' => array(),
			'convert' => self::$conversations,
		);
		// List stored views
		foreach(array_keys($views) as $app_name)
		{
			list($app, $name) = explode(':', $app_name);
			$sel_options['view'][$app_name] = lang($app).': '.$name;
		}
		// plus default view of apps
		foreach(array_keys(self::$apps) as $app)
		{
			$sel_options['view'][$app] = lang($app).' ('.lang('Default').')';
		}
		$tpl = new Etemplate('stylite.view');
		$tpl->exec('stylite.stylite_view.generator', $content, $sel_options, array(), array(
			'old_view' => $content['view'],
			'defs' => $content['defs'],
		));
	}

	/**
	 * Get default view for an app
	 *
	 * @param string $app
	 * @param boolean $enable =true value for "enable" attribute
	 * @return array
	 */
	private static function default_view($app, $enable=true)
	{
		$app_data = self::$apps[$app];
		$main = $GLOBALS['egw']->db->get_table_definitions(
			isset($app_data['app']) ? $app_data['app'] : $app, $app_data['main']);

		$definition = array();
		foreach($main['fd'] as $col => $def)
		{
			$def['column'] = $col;
			$def['enable'] = $enable;
			// implicit conversation defaults
			switch($def['type'].$def['precision'])
			{
				case 'int8':
					$def['convert'] = 'timestamp';
					break;

				case 'int4':
				default:
					if (preg_match('/(^account_id|_(modifier|creator|owner|responsible))$/', $col))
					{
						$def['convert'] = 'account';
					}
					elseif (preg_match('/(^cat_id|_cat(egory|s)?)$/', $col))
					{
						$def['convert'] = 'cat';
					}
					// for varchar we show 3 values by default
					if ($def['type'] == 'varchar') $def['convert'] .= '3';
					break;
			}
			// explict conversation default given --> use it not implicit ones
			if (isset($app_data['convert']) && isset($app_data['convert'][$col]))
			{
				$def['convert'] = $app_data['convert'][$col];
			}
			$definition[$col] = $def;
		}
		// add custom fields
		if (isset($app_data['extra']) && ($cfs = Api\Storage\Customfields::get($app)))
		{
			//_debug_array($cfs);
			foreach($cfs as $name => $cf)
			{
				// generate a valid column-name for the cf
				$col = strtolower(preg_replace('/[^A-Za-z0-9_]+/', '', $name));
				$n = null;
				while (self::find($definition, $col.$n) !== false)
				{
					++$n;
				}
				$col .= $n;
				$def = array(
					'column' => '#'.$name,
					'type' => $cf['type'],
					'rename' => $col,
					'comment' => $cf['label'],
					'enable' => $enable,
				);
				// some default conversation depending on cf type
				switch($cf['type'])
				{
					case 'select':
					case 'radio':
						$def['convert'] = 'cf-label';
						break;
					case 'select-account':
						$def['convert'] = 'account';
						break;
				}
				$definition['#'.$name] = $def;
			}
		}
		//_debug_array($definition);

		return $definition;
	}

	/**
	 * Find a name in definition array
	 *
	 * @param array $definition
	 * @param string $name
	 * @param int|bool $not_idx index NOT to match, or false
	 * @return int|bool integer index or false, if not found
	 */
	private static function find(array $definition, $name, $not_idx=false)
	{
		$ret = false;
		foreach($definition as $idx => $data)
		{
			if ((!empty($data['rename']) ? $data['rename'] : $data['column']) == $name &&
				($not_idx === false || $not_idx != $idx))
			{
				$ret = $idx;
				break;
			}
		}
		//error_log(__METHOD__."(..., '$name', ".array2string($not_idx).') returning '.array2string($ret));
		return $ret;
	}

	/**
	 * Generate SQL to create or update a view based on given definition
	 *
	 * @param string $name name of view
	 * @param array $definition
	 * @param string $app
	 * @return string
	 */
	private static function sql_view($name, array $definition, $app)
	{
		$app_data = self::$apps[$app];
		$joins = $cols = array();
		foreach($definition as $def)
		{
			if (!$def || !$def['enable']) continue;

			// custom field
			if ($def['column'][0] == '#')
			{
				$table_alias = $GLOBALS['egw']->db->name_quote($def['rename']);
				$joins[] = 'LEFT JOIN '.$app_data['extra'].' '.$table_alias.' ON '.
					$app_data['main'].'.'.$app_data['id_col'].'='.$table_alias.'.'.$app_data['id_col'].' AND '.
					$table_alias.'.'.$app_data['extra_name_col'].'='.$GLOBALS['egw']->db->quote(substr($def['column'], 1));
				$column = $table_alias.'.'.$app_data['extra_value_col'];
				$org_column = $def['rename'];
			}
			else	// regular column
			{
				$column = $app_data['main'].'.'.$def['column'];
				$org_column = $def['column'];
			}

			// handle nth element in comma-separated list, 	eg. cat2 for 2. cat in comma-sep. list
			if (($num=(int)substr($def['convert'], -1)) > 1)
			{
				$def['convert'] = substr($def['convert'], 0, -1);
			}
			else
			{
				$num = 1;
			}
			for ($n = 1; $n <= $num; ++$n)
			{
				$col = self::nth_in_set($n, $column);
				$org_col = ($def['rename'] ? $def['rename'] : $org_column).($n > 1 ? $n : '');

				switch($def['convert'])
				{
					case 'account':
						$table_alias = $GLOBALS['egw']->db->name_quote(str_replace('.', '_', $org_col));
						$joins[] = 'LEFT JOIN egw_accounts '.$table_alias.' ON ABS('.$col.')='.$table_alias.'.account_id';
						$col = $table_alias.'.account_lid';
						break;

					case 'cat':
						$table_alias = $GLOBALS['egw']->db->name_quote(str_replace('.', '_', $org_col));
						$joins[] = 'LEFT JOIN egw_categories '.$table_alias.' ON '.$col.'='.$table_alias.'.cat_id';
						$col = $table_alias.'.cat_name';
						break;

					case 'timestamp':
						$col = 'CASE '.$col.' WHEN 0 THEN NULL ELSE '.$GLOBALS['egw']->db->from_unixtime($col).' END';
						break;

					case 'cf-label':
						$col = self::convert_cf_label($def['column'], $col, $app);
						break;
				}
				if ($col != $org_col)
				{
					$col .= ' AS '.$GLOBALS['egw']->db->name_quote($org_col);
				}
				$cols[] = $col;
			}
		}
		return 'CREATE OR REPLACE VIEW '.$GLOBALS['egw']->db->name_quote($name).' AS SELECT'.
			"\n  ".implode(",\n  ", $cols)."\nFROM ".$app_data['main'].
			($joins ? "\n".implode("\n", $joins) : '');
	}

	/**
	 * Replace values of cf select-boxes or radio-buttons with their label
	 *
	 * If used on non-cf or cf of other type then select or radio, conversation get silently ignored!
	 *
	 * @param string $cf cf-name includeing #-prefix
	 * @param string $col sql expression for the value
	 * @returns string sql
	 */
	private static function convert_cf_label($cf, $col, $app)
	{
		if ($cf[0] != '#' || !($cfs = Api\Storage\Customfields::get($app)) ||
			!($def=$cfs[substr($cf, 1)]) || !in_array($def['type'], array('select', 'radio')))
		{
			return $col;
		}
		$db = $GLOBALS['egw']->db;
		$sql = 'CASE ('.$col.')';
		foreach($def['values'] as $value => $label)
		{
			$sql .= ' WHEN '.$db->quote($value).' THEN '.$db->quote($label);
		}
		$sql .= ' END';

		return $sql;
	}

	/**
	 * Get nth element out of a comma-separated list using SQL
	 *
	 * nth_in_set(1, $col="1,2,3") returns "1,2,3"
	 * nth_in_set(2, $col="1,2,3") returns "2,3"
	 * nth_in_set(4, $col="1,2,3") returns NULL
	 *
	 * @param int $n 1, ...
	 * @param string $col column name
	 * @param string $sep =','
	 * @return string with sql
	 */
	public static function nth_in_set($n, $col, $sep=',')
	{
		if ($n <= 1) return $col;

		$start = 1;
		while ($n-- > 2)
		{
			$start = "1+LOCATE('$sep',$col,$start)";
		}
		return "CASE LOCATE('$sep',$col,$start) WHEN 0 THEN NULL ELSE SUBSTR($col,1+LOCATE('$sep',$col,$start)) END";
	}
}
