<?php
/**
 * EPL Functions - Report for tracker
 *
 * @package epl
 * @subpackage tracker
 * @author Nathan Gray
 * @copyright 2011 Nathan Gray
 * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
 * @version $Id$
 */

use EGroupware\Api;

class tracker_report extends stylite_jqplot_report {

	protected $group_by_options = array(
		'cat_id'	=> 'category',
		'tr_status'	=> 'status',
		'tr_resolution'	=> 'resolution',
		'tr_version'	=> 'version',
		'tr_creator'	=> 'created by',
		'tr_assigned'	=> 'assigned to'
	);

	protected $current_table = 'egw_tracker';

	protected $date_field = null;

	protected $current_colors = array();

	protected $type_field = 'tr_tracker';

	/**
	 * Constructor - don't use, it's all static, but framework needs to instanciate parent
	 */
	public function __construct() {
		$GLOBALS['egw_info']['flags']['currentapp'] = 'tracker';
		parent::__construct();

		foreach($this->group_by_options as &$value) {
			$value = ucfirst(lang($value));
		}
		$ui = new tracker_ui();
		$this->field2history = $ui->field2history;
		$this->field2label = $ui->field2label;

		$this->select_options = array(
			'tr_tracker'	=> $ui->trackers,
			'tr_version'	=> $ui->get_tracker_labels('version', null),
			'tr_status'	=> $ui->get_tracker_stati(null),
			'tr_resolution'	=> $ui->get_tracker_labels('resolution',null),
			'cat_id'	=> $ui->get_tracker_labels('cat',null),
		);

		// Date is opened date, which is stored as a unix timestamp
		$this->date_field = $GLOBALS['egw']->db->from_unixtime('tr_created');
	}

	/**
	 * Return a singleton instance, to be PHP v5.2 complient & as static as possible
	 */
	public static function instance() {
		if(tracker_report::$instance == null) {
			tracker_report::$instance = new tracker_report();
		}
		return tracker_report::$instance;
	}

	/**
	 * Process parameters interactively for report
	 *
	 * Extended to get tracker specific labels for version, category, etc.
	 *
	 * @param content Incoming data from form
	 * @param data Output data for etemplate
	 * @param sel_options Selectbox options
	 * @param readonlys
	 */
	public static function get_parameters(Array &$content = array(), Array &$data = array(), Array &$sel_options = array(), Array &$readonlys = array()) {
		unset($data, $readonlys);	// unused

		$ui = new tracker_ui();
		if($content['tr_tracker']) {
			$tr_tracker = array($content['tr_tracker']);
		} else {
			$tr_tracker = array_keys($ui->get_tracker_labels());
		}
		$instance =& self::instance();
		if(!isset($sel_options['tr_status'])) $sel_options = $instance->select_options;
		$sel_options['tr_tracker'] = $ui->trackers;
		foreach($tr_tracker as $tracker_id) {
			$sel_options['tr_version'] += @array_diff_key($ui->get_tracker_labels('version', $tracker_id),$sel_options['tr_version']);
			$sel_options['tr_status'] += @array_diff_key($ui->get_tracker_stati($tracker_id), $sel_options['tr_status']);
			$sel_options['tr_resolution'] += @array_diff_key($ui->get_tracker_labels('resolution', $tracker_id),$sel_options['tr_resolution']);
			$sel_options['cat_id'] += @array_diff_key($ui->get_tracker_labels('cat', $tracker_id),$sel_options['cat_id']);
		}
		$instance->select_options = array_merge($instance->select_options, $sel_options);

		return $content;
	}

	/**
	 * Get data for current settings
	 *
	 * Graph is made up of multiple pie charts showing the current breakdown.
	 * One pie for Status, Category, Version, Created by, Assigned to, and the first
	 * (if any) valid custom field.
	 *
	 * @param parameters Filters to filter data
	 * @param translate_field If fields need to be translated
	 * @return Array of (pure) data
	 */
	public static function get_current(Array $parameters, $translate_fields = true) {
		$data = array();

		$tracker = new tracker_bo();
		$series_list = array('tr_status', 'cat_id', 'tr_version', 'tr_creator','tr_assigned');

		unset($parameters['time_frame']);
		unset($parameters['app']);
		unset($parameters['end']);
		unset($parameters['group_by']);
		if(!$parameters['tr_tracker']) $parameters['tr_tracker'] = array_keys($tracker->get_tracker_labels());
		if(is_array($parameters['cf'])) {
			foreach($parameters['cf'] as $custom => $value) {
				$parameters['#'.$custom] = $value;
			}
		}
		unset($parameters['cf']);

		// Load parameters for selected tracker
		foreach((array)$parameters['tr_tracker'] as $tracker_id) {
			$t= array('tr_tracker' => $tracker_id);
			self::instance()->get_parameters($t);
		}

		// Find custom fields that are selectbox or radio
		$custom = Api\Storage\Customfields::get('tracker',false, is_array($parameters['tr_tracker']) ? null: $parameters['tr_tracker']);
		$extra = array();
		foreach($custom as $cf_name => $cf) {
			if($cf['type'] == 'select' || $cf['type'] == 'radio') {
				self::$instance->field2label[$cf_name] = $cf['label'];
				$extra[$cf_name] = $cf['values'];
				$extra[$cf_name][''] = lang('None');
				$series_list[] = $cf_name;
			}
		}

		// Get matching entries
		// query is undefined
		$rows = $tracker->search($query, false, '','','',false,'AND',false,$parameters);
		foreach((array)$rows as $row) {

			$custom = $tracker->read_customfields($row['tr_id'], array_keys($extra));
			foreach($series_list as $series) {
				$title = $translate_fields ? lang(self::$instance->field2label[$series]) : $series;
				$name = !$row[$series] ? lang('None') : lang(self::$instance->select_options[$series][$row[$series]]);
				if($series == 'tr_creator' || $series == 'tr_assigned') {
					if($row[$series] && is_array($row[$series])) {
						foreach($row[$series] as $name) {
							$name = $row[$series] == null ? lang('None') : Api\Accounts::username($name);
							$data[$title][$name]++;
						}
						continue;
					} else {
						$name = ($row[$series] == null || !$row[$series]) ? lang('None') : Api\Accounts::username($row[$series]);
					}
				} elseif ($extra[$series]) {
					$name = $extra[$series][$custom[$row['tr_id']]['#'.$series]];
				}

				// Fallback in case the category can't be found
				if(!$name) $name = $GLOBALS['egw']->categories->id2name($row[$series]);

				$data[$title][$name]++;
			}
		}
		return $data;
	}

	/**
	 * Get aged report showing summary by opened date
	 *
	 * This gets current entries, for display by how old they are.
	 *
	 * @param time_slice How finely to slice the time - monthly, weekly, etc.
	 * @param start Earliest date in the graph
	 * @param end Latest date in the graph
	 * @param filters List of field filters
	 * @param group_by How to aggregate the data
	 *
	 * @return array of data for further use
	 */
	public static function get_aged($parameters) {
		$instance = self::instance();

		if(!$parameters['tr_tracker']) {
			$tracker = new tracker_bo();
			$parameters['tr_tracker'] = array_keys($tracker->get_tracker_labels());
		}

		// Load parameters for selected tracker
		foreach((array)$parameters['tr_tracker'] as $tracker_id) {
			$t= array('tr_tracker' => $tracker_id);
			$instance->get_parameters($t);
		}

		// Add what it takes for custom fields
		$custom = Api\Storage\Customfields::get('tracker');
		if($parameters['group_by'] == 'tr_assigned' || in_array('tr_assigned', array_keys($parameters))) {
			$parameters['join'] .= 'LEFT JOIN egw_tracker_assignee ON egw_tracker.tr_id = egw_tracker_assignee.tr_id ';
		}
		if ($custom && (in_array($parameters['group_by'], array_keys($custom)))) {
			$parameters['join'] .= 'LEFT JOIN egw_tracker_extra ON egw_tracker.tr_id = egw_tracker_extra.tr_id AND egw_tracker_extra.tr_extra_name = ' . $GLOBALS['egw']->db->quote($parameters['group_by']);
			$instance->select_options[$parameters['group_by']] = $custom[$parameters['group_by']]['values'];
			$parameters['group_by'] = 'egw_tracker_extra.tr_extra_value';
		}
		if(is_array($parameters['cf']) && array_intersect_key($custom, $parameters['cf'])) {
			foreach($parameters['cf'] as $c_name => $c_value) {
				$q_c_name = $GLOBALS['egw']->db->quote($c_name);
				$q_c_value = $GLOBALS['egw']->db->quote($c_value);

				$parameters['join'] .= 'JOIN egw_tracker_extra as ' . $GLOBALS['egw']->db->name_quote($c_name). ' ON egw_tracker.tr_id = ' . $GLOBALS['egw']->db->name_quote($c_name) . ".tr_id AND ". $GLOBALS['egw']->db->name_quote($c_name) .".tr_extra_name = $q_c_name  AND " . $GLOBALS['egw']->db->name_quote($c_name).".tr_extra_value=$q_c_value";
			}
		}
		if(is_array($custom) && is_array($parameters['cf']) &&
			(in_array($parameters['group_by'], array_keys($custom)) || array_intersect_key($custom, $parameters['cf']))
		) {
			$instance->field2label += $parameters['cf'];
		}

		// Pass off to parent for actual querying & data sanitizing
		$data = parent::get_aged($parameters);

		// Set account IDs for graph generation
		foreach(array_keys($data) as $series) {
			if(in_array($parameters['group_by'], array('tr_creator', 'tr_assigned')))
			{
				$instance->select_options[$parameters['group_by']][$series] = $series == '' ? lang('None') : Api\Accounts::username($series);
			} elseif (!in_array($series, array_keys($instance->select_options[$parameters['group_by']]))) {
				$instance->select_options[$parameters['group_by']][$series] = ($series =='' ? lang('None') : $GLOBALS['egw']->categories->id2name($series));
			}
		}
		return $data;
	}

	/**
	 * Set graph subtitle based on filter parameters
	 * @param parameters Array of filters used to select data
	 */
	protected static function set_subtitle(&$graph, $_parameters) {
		parent::set_subtitle($graph, $_parameters);

		$subtitle = array();

		// Just remove anything with no value
		foreach($_parameters as $field => $value) {
			if(!$value) unset($_parameters[$field]);
		}

		// Abuse export converter to do human friendly conversion
		$record = new tracker_egw_record();
		$record->set_record($_parameters);
		importexport_export_csv::convert($record, tracker_egw_record::$types, 'tracker', self::$instance->select_options);
		$parameters = $record->get_record_array();

		// Build subtitle
		$bo = new tracker_bo();
		foreach($parameters as $field => $value) {
			if($value && !in_array($field, array('app','time_frame', 'time_slice', 'start', 'end', 'group_by', 'cf'))) {
				$subtitle[] = ($bo->field2label[$field] ? $bo->field2label[$field] : $field) . ': ' . $value;
			}
			if($value && $field == 'cf') {
				$cfs = Api\Storage\Customfields::get('tracker');
				foreach($value as $c_name => $c_field) {
					if($c_field) $subtitle[] = $cfs[$c_name]['label'] . ': ' . $cfs[$c_name]['values'][$c_field];
				}
			}
		}
		if($subtitle) {
			if(is_array($graph)) $graph['subtitle'] = implode(', ', $subtitle);
		}
	}
}

tracker_report::static_init();
