/**
 * EPL Functions - Projectmanager - Javascript Stylite extensions
 *
 * @link http://www.egroupware.org
 * @package epl
 * @subpackage projectmanager
 * @author Nathan Gray
 * @copyright (c) 2015 by Nathan Gray
 * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
 * @version $Id$
 */

/**
 * UI for projectmanager
 *
 * @augments AppJS
 */
app.classes.stylite_gantt =
{
	// These fields are what get changed (see class.stylite_projectmanager_gantt.inc.php)
	// and in what edit mode they are changed.  old_field is from the server (old),
	// a timestamp for dates. gantt_field is from the gantt chart, formatted according
	// to gantt widget's config settings (Y-m-d H:i:s)
	edit_fields: {
		calendar: [
			{label: 'Start date', old_field: 'pe_start', gantt_field: 'start_date', modes: ['move','resize']},
			{label: 'End date',   old_field: 'pe_end',   gantt_field: 'end_date', modes: ['move', 'resize']},
		],
		infolog: [
			{label: 'Status',  old_field: 'pe_completion', gantt_field: 'progress', modes: ['progress']},
			{label: 'Start date', old_field: 'pe_start',      gantt_field: 'start_date', modes: ['move','resize']},
			{label: 'End date',   old_field: 'pe_end',        gantt_field: 'end_date', modes: ['move','resize']}
		],
		tracker: [
			{label: 'Status',  old_field: 'pe_completion', gantt_field: 'progress', modes: ['progress']},
			{label: 'Start date', old_field: 'pe_start',      gantt_field: 'start_date', modes: ['move','resize']},
			{label: 'End date',   old_field: 'pe_end',        gantt_field: 'end_date', modes: ['move','resize']},
		],
		projectmanager: [
			{label: 'Status',  old_field: 'pm_completion', gantt_field: 'progress', modes: ['progress']},
			{label: 'Start date', old_field: '--',      gantt_field: 'start_date', modes: ['move','resize']},
			{label: 'End date',   old_field: '--',        gantt_field: 'end_date', modes: ['move','resize']},
		]

	},

	/**
	 * Change callback for gantt chart.
	 *
	 * @param {Event} e Mode, task and value are passed in e.data
	 * @param {et2_gantt} widget
	 * @returns {Boolean} Continue with normal ajax update
	 */
	change: function(e, widget)
	{
		var fields = [];
		var task = e && e.data ? e.data.task || {} : false;
		if(!task) return false;

		var values = e.data.value || {};
		var _app = task.pe_app ? task.pe_app : (task.pm_creator ? 'projectmanager' : '')
		var remember_preference_key = 'gantt_update_'+e.data.mode+'_'+app;

		// Projects themselves need more work - there's no project element
		this.edit_fields.projectmanager[1].old_field = (values.planned_times == "true" ? 'pm_planned' : 'pm_real') + '_start';
		this.edit_fields.projectmanager[2].old_field = (values.planned_times == "true" ? 'pm_planned' : 'pm_real') + '_end';

		if(this.edit_fields[_app])
		{
			var app_fields = this.edit_fields[_app]|| {};
			for(var field in app_fields)
			{
				if(app_fields && app_fields[field].modes.indexOf(e.data.mode)!= -1)
				{
					var values = {
						label: widget.egw().lang(app_fields[field].label),
						old_value: task[app_fields[field].old_field],
						new_value: task[app_fields[field].gantt_field]
					};
					if(app_fields[field].gantt_field.indexOf('date') != -1)
					{
						// Reformat dates from server-friendly timestamp to user format
						values.old_value = date(egw.preference('dateformat') + " " +
							(egw.preference('timeformat') == '24' ? 'H:i' : 'g:i a'),values.old_value);
						values.new_value = date(egw.preference('dateformat') + " " +
							(egw.preference('timeformat') == '24' ? 'H:i' : 'g:i a'),new Date(values.new_value));
					}
					if(app_fields[field].gantt_field == 'progress')
					{
						values.new_value = parseInt(values.new_value * 100) + '%'
					}
					if(values.old_value != values.new_value)
					{
						fields.push(values);
					}
				}
			}
		}
		// No fields to update, no point in asking.  Let normal procedure take over.
		if(fields.length == 0) return true;

		// Apply previous answer, if there is one
		if(egw.preference(remember_preference_key, 'projectmanager') == et2_dialog.YES_BUTTON)
		{
			e.data.value['datasource_update'] = true;
			app.classes.stylite_gantt.update(widget, e.data.task, e.data.mode, e.data.value);
			return false;
		}
		else if(egw.preference(remember_preference_key, 'projectmanager') == et2_dialog.NO_BUTTON)
		{
			// Don't update anything
			return false;
		}
		var remember = jQuery("<label style='margin:1em' class='ui-button ui-widget ui-corner-all ui-button-text-only'><input type='checkbox'/>"+widget.egw().lang('Remember and do not ask again')+"</label>");
		var dialog = et2_createWidget("dialog",{
			// If you use a template, the second parameter will be the value of the template, as if it were submitted.
			callback: function(button_id, value) {
				if(jQuery(':checked',remember).length)
				{
					// Remember for next time
					egw.set_preference('projectmanager', remember_preference_key, button_id);
				}
				if(button_id == et2_dialog.YES_BUTTON)
				{
					// Update datasource
					e.data.value['datasource_update'] = true;
					app.classes.stylite_gantt.update(widget, e.data.task, e.data.mode, e.data.value);
				}
				else if (button_id == et2_dialog.NO_BUTTON)
				{
					// Cancel and restore
					widget.refresh();
					this.div.dialog("close");
					// Don't update anything
					return false;
				}
			},
			buttons: et2_dialog.BUTTONS_YES_NO,
			title: widget.egw().lang('Update %1 (%2)',widget.egw().lang(e.data.task.pe_app||''), widget.egw().lang(e.data.mode)||''),
			template:"projectmanager.gantt.edit",
			value: { content: {fields: fields || [], pe_title: task.text.trim()}, sel_options: {}, currentapp:'projectmanager'}
		});

		// Add the checkbox
		dialog.div.parent().find(".ui-dialog-buttonset").append(remember);

		// Don't do the normal change handler
		return false;
	},

	/**
	 * Calls the gantt widget's update callback, and triggers a refresh after
	 *
	 * @param {et2_gantt} gantt_widget
	 * @param {Object} task
	 * @param {String} mode
	 * @param {Object} value
	 */
	update: function(gantt_widget, task, mode, value)
	{
		var request = gantt_widget.egw().json(gantt_widget.options.ajax_update,
			[task, mode, value], function() { gantt_widget.refresh(this.parent);},{parent: task.parent||0}
		).sendRequest(true);
	}
};