/**
 * EPL Functions - Group-Admin - Javascript UI
 *
 * @link http://www.egroupware.org
 * @package epl
 * @subpackage infolog
 * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
 * @copyright (c) 2015 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
 * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
 * @version $Id$
 */

/**
 * InfoLog encryption
 *
 * Get only loaded in InfoLog edit popup!
 *
 * @augments AppJS
 */
app.classes.stylite = AppJS.extend(
/**
 * @lends app.classes.stylite
 */
{
	appname: 'stylite',

	/**
	 * Constructor
	 *
	 * @memberOf app.classes.stylite
	 */
	init: function()
	{
		// call parent
		this._super.apply(this, arguments);
	},

	/**
	 * Destructor
	 */
	destroy: function()
	{
		// call parent
		this._super.apply(this, arguments);
	},

	/**
	 * Toggle encryption
	 *
	 * @param {jQuery.Event} _event
	 * @param {et2_button} _widget
	 * @param {DOMNode} _node
	 */
	toggleEncrypt: function(_event, _widget, _node)
	{
		var self = this;

		// switching encryption on
		if (!jQuery(_node).hasClass('toolbar_toggled'))
		{
			if (typeof mailvelope == 'undefined')
			{
				self.mailvelopeInstallationOffer();
				// switch encrypt button off again
				return;
			}

			// check if we have keys for all recipents, before switching
			this.mailvelopeGetCheckRecipients().then(function(_recipients)
			{
				self.mailvelopeOpenKeyring().then(function(_keyring)
				{
					jQuery(_node || 'button#infolog-edit_encrypt').toggleClass('toolbar_toggled');
					self.mailvelopeCompose(_keyring);
				});
			})
			.catch(function(_err)
			{
				self.egw.message(_err.message, 'error');
			});
		}
		else
		{
			// switch Mailvelop off again, but warn user he will loose his content
			et2_dialog.show_dialog(function (_button_id)
			{
				if (_button_id == et2_dialog.YES_BUTTON )
				{
					self.et2.getWidgetById('info_des').set_disabled(false);
					jQuery(self.mailvelope_iframe_selector).remove();
					jQuery(_node).removeClass('toolbar_toggled');
				}
			},
			this.egw.lang('You will loose current message body, unless you save it to your clipboard!'),
			this.egw.lang('Switch off encryption?'),
			{}, et2_dialog.BUTTON_YES_NO, et2_dialog.WARNING_MESSAGE, undefined, this.egw);
		}
	},

	/**
	 * Editor object of active compose
	 *
	 * @var {Editor}
	 */
	mailvelope_editor: undefined,

	/**
	 * Called on compose, if mailvelope is available
	 *
	 * @param {Keyring} _keyring Mailvelope keyring to use
	 */
	mailvelopeCompose: function(_keyring)
	{
		delete this.mailvelope_editor;

		var editor = this.et2.getWidgetById('info_des');
		var options = { predefinedText: editor.get_value() };

		// check if we have some sort of reply to an encrypted message
		// --> parse header, encrypted mail to quote and signature so Mailvelope understands it
		var start_pgp = options.predefinedText.indexOf(this.begin_pgp_message);
		if (start_pgp != -1)
		{
			var end_pgp = options.predefinedText.indexOf(this.end_pgp_message);
			if (end_pgp != -1)
			{
				options = {
					quota: 64,	// is only text = 64k in database
					quotedMailHeader: start_pgp ? options.predefinedText.slice(0, start_pgp).replace(/> /mg, '').trim()+"\n" : "",
					quotedMail: options.predefinedText.slice(start_pgp, end_pgp+this.end_pgp_message.length+1).replace(/> /mg, ''),
					quotedMailIndent: start_pgp != 0,
					predefinedText: options.predefinedText.slice(end_pgp+this.end_pgp_message.length+1).replace(/^> \s*/m,''),
					keepAttachments: true
				};
				// set encrypted checkbox, if not already set
				jQuery(this.et2.getWidgetById('encrypt').getDOMNode()).addClass('toolbar_toggle');
			}
		}

		var self = this;
		mailvelope.createEditorContainer('div#infolog-edit_infolog-edit-description', _keyring, options).then(function(_editor)
		{
			self.mailvelope_editor = _editor;
			// install our onSubmit handler
			var submit = self.et2._inst.submit;
			self.et2._inst.submit = function() {
				var args = arguments;
				if (self.mailvelopeOnSubmit.call(self, function()
				{
					return submit.apply(self.et2._inst, args);
				}) !== false)
				{
					return submit.apply(self.et2._inst, arguments);
				}
			};
			editor.set_disabled(true);
		},
		function(_err)
		{
			self.egw.message(_err.message, 'error');
		});
	},

	/**
	 * OnSubmit callback to inject encrypted message in info_des widget
	 *
	 * @param {function} _submit callback to call to submit
	 * @returns {Boolean}
	 */
	mailvelopeOnSubmit: function(_submit)
	{
		if (this.mailvelope_editor)
		{
			var self = this;
			this.mailvelopeGetCheckRecipients().then(function(_recipients)
			{
				return self.mailvelope_editor.encrypt(_recipients);
			}).then(function(_armored)
			{
				self.et2.getWidgetById('info_des').set_disabled(false);
				self.et2.getWidgetById('info_des').set_value(_armored);
				_submit();
			}).catch(function(_err)
			{
				self.egw.message(_err.message, 'error');
			});
			return false;
		}
	},

	/**
	 * Check if we have a key for all recipients
	 *
	 * @returns {Promise.<Array, Error>} Array of recipients or Error with recipients without key
	 */
	mailvelopeGetCheckRecipients: function()
	{
		// collect all recipients
		var recipients = [this.et2.getWidgetById('info_owner').options.value];
		recipients = recipients.concat(this.et2.getWidgetById('info_responsible').get_value());

		var self = this;
		return new Promise(function(_resolve, _reject)
		{
			self.egw.accountData(recipients, 'account_email', true, function(_emails)
			{
				_resolve(self._super.call(self, jQuery.map(_emails, function(_email, _account_id) { return _email; })));
			});
		});
	},

	/**
	 * OnChange callback for responsible
	 *
	 * @param {jQuery.Event} _event
	 * @param {et2_widget} _widget
	 */
	onchangeResponsible: function(_event, _widget)
	{
		var self = this;
		this.mailvelopeGetCheckRecipients().catch(function(_err)
		{
			self.egw.message(_err.message, 'error');
		});
	},

	/**
	 * Enable decrypted tooltip on the given widget
	 *
	 * @param {type} widget
	 * @param {string} selector	dom selector
	 *
	 * @returns {undefined}
	 */
	decrypt_hover: function(widget,selector)
	{
		var self = this;
		if(typeof selector === 'undefined')
		{
			selector = '.infoDes';
		}
		var tooltip = jQuery(widget.getDOMNode(widget))
			.tooltip({
				items: selector+':contains("'+this.begin_pgp_message+'")',
				show: {
					// Decrypting can take a bit, so delay show a little more than normal
					//effect: 'blind',
					delay: 1000
				},
				position:{at:"left top",collision:"none"},
				content: function() {
					var text = jQuery(this).html();
					text = text.slice(text.indexOf(self.begin_pgp_message));
					return text.replace(/<br\s*\/?>/mg,"\n");
				},
				open: function(event,ui)
				{
					try
					{
						var width = jQuery(event.originalEvent.target).width()+'px';
						ui.tooltip.css('max-width','inherit')
							.width(width)
							.height('245px');
						self.mailvelopeOpenKeyring().then(function(_keyring)
						{
							var text = jQuery('.ui-tooltip-content',ui.tooltip).height('inherit')
								.html().replace(/<br\s*\/?>/mg,"\n");

							ui.tooltip.children().empty();
							mailvelope.createDisplayContainer('#'+ui.tooltip.attr('id')+' > div',text,_keyring)
								.then(function() {},function() {tooltip.tooltip('close');});
						});
					} catch(e){}
				},
				close: function( event, ui ) {

					ui.tooltip.hover(
						function () {
							jQuery(this).stop(true).fadeTo(1, 1);
						},
						function () {
							jQuery(this).fadeOut("400", function(){ jQuery(this).remove(); })
						}
					);
				}
			});
	}
});
