/**
*	Javascript for doing a link search across all applications
*
*	@author Nathan Gray
*/

var link_search = {
	/**
	*	How many applications we are searching?
	*	We'll count as we go through them, and when the last response comes in,
	*	hide the "Please wait..." message.
	*/
	app_search_count: 0,

	/**
	*	Go through each application that is set up and run the query using AJAX.
	*	The response contains the code needed to display the results.
	*
	*	@param pattern Text to search for
	*/
	search: function(widget) {

		this.app_search_count = 0;

		var pattern = widget.getParent().getWidgetById('query').getValue();

		// Go through list of applications that support linking
		widget.getRoot().getWidgetById('apps').iterateOver(function(app) {

			// Call query - once for each app, as options may be different
			egw.json('stylite_link_search_ui::ajax_query', [
				'',
				app.dom_id, app.id, pattern, {'type' : app.getArrayMgr('content').getEntry('type')}
				], link_search.results, app, true, app
			).sendRequest(true);

			app.getParent().set_disabled(true);
			this.app_search_count++;

			// Update links
			var entry = app.getArrayMgr('content').data;
			if(entry && entry.list) {
				var link = app.getWidgetById('more_link');
				link.onclick = function() {
					egw.open('',entry.app, 'list', {search: pattern},entry.app);
					return false;
				};
			}
		},this,et2_template);

		if(this.app_search_count > 0) {
			this.show_wait(true);
		}
	},

	results: function results(data)
	{
		if(data.type) return;
		this.getWidgetById('results').set_value(data);

		this.getParent().set_disabled(!data || data.length == 0);
		link_search.app_search_count--;
		if(link_search.app_search_count <= 0) link_search.show_wait(false);
	},

	/**
	*	Show or hide a 'Please wait...' message
	*/
	show_wait: function(display) {
		document.getElementById('stylite-link_search-search_wait').style.display = (display ? 'block' : 'none');
	},

	/**
	*	Show or hide a 'More...' link
	*/
	show_more: function(app, display) {
		var link = etemplate2.getById("stylite-link_search-search").widgetContainer
			.getWidgetById(app).getWidgetById('more_link');
		link.set_disabled(!display);
	}
};