<?php
/**
 * EPL: VFS - stream wrapper for linked files with just versioning
 *
 * @link http://www.egroupware.org
 * @package epl
 * @subpackage blinks
 * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
 * @copyright (c) 2008-18 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
 */

/**
 * Use Stylite\Vfs\Versioning\StreamWrapper as parent for the Api\Vfs\Links\StreamWrapper to enable versioning
 */
namespace EGroupware\Api\Vfs\Links;

if (!class_exists('EGroupware\\Api\\Vfs\\Links\\LinksParent', false))
{
	class LinksParent extends \EGroupware\Stylite\Vfs\Versioning\StreamWrapper {}
}

/**
 * Our own namespace in EPL app
 */
namespace EGroupware\Stylite\Vfs\Vlinks;

use EGroupware\Api\Vfs;

/**
 * EPL: stream wrapper for linked files with just versioning
 *
 * The files stored by the Vfs\Sqlfs\StreamWrapper in a /apps/$app/$id directory
 *
 * The links stream wrapper extends the sqlfs one, to implement an own ACL based on the access
 * of the entry the files are linked to.
 *
 * Applications can define a 'file_access' method in the link registry with the following signature:
 *
 * 		boolean function file_access(string $id,int $check,string $rel_path)
 *
 * If the do not implement such a function the title function is used to test if the user has
 * at least read access to an entry, and if true full (write) access to the files is granted.
 *
 * The stream wrapper interface is according to the docu on php.net
 *
 * @link http://www.php.net/manual/en/function.stream-wrapper-register.php
 */
class StreamWrapper extends Vfs\Links\StreamWrapper
{
	/**
	 * Scheme / protocoll used for this stream-wrapper
	 */
	const SCHEME = 'stylite.vlinks';
	/**
	 * Prefix to predend to get an url from a path
	 */
	const PREFIX = 'stylite.vlinks://default';
	/**
	 * Base url to applicaton data
	 */
	const BASEURL = 'stylite.vlinks://default/apps';

	/**
	 * Init our static properties
	 */
	static public function init_static()
	{
		// register our scheme
		stream_register_wrapper(self::SCHEME ,__CLASS__);
	}
}

StreamWrapper::init_static();
